<?php

namespace Drupal\webform_booking\Plugin\WebformElement;

use Drupal\Core\Form\FormStateInterface;
use Drupal\webform\Plugin\WebformElementBase;
use Drupal\webform\WebformSubmissionInterface;

/**
 * Provides a 'webform_booking' element.
 *
 * @WebformElement(
 *   id = "webform_booking",
 *   label = @Translation("Booking"),
 *   description = @Translation("Provides a webform element for scheduling appointments."),
 *   category = @Translation("Booking"),
 *   states_wrapper = TRUE,
 * )
 *
 * @FormElement("webform_webform_booking")
 *
 * @see \Drupal\Core\Render\Element\FormElement
 * @see \Drupal\webform\Plugin\WebformElementBase
 */
class WebformBooking extends WebformElementBase {

  /**
   * {@inheritdoc}
   */
  protected function defineDefaultProperties() {
    $properties = parent::defineDefaultProperties() + [
      'title' => '',
      'start_date' => '',
      'end_date' => '',
      'exclusion_dates' => '',
      'excluded_weekdays' => [],
      'time_interval' => '',
      'slot_duration' => '',
      'seats_slot' => '1',
    ];
    unset($properties['markup']);
    return $properties;
  }

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {
    $form = parent::form($form, $form_state);

    // Hide/Remove default value, form display and markup form elements.
    unset($form['markup'], $form['default']);
    $form['form']['#access'] = FALSE;

    $form['appointment_settings'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Webform Booking Settings'),
    ];

    $form['appointment_settings']['title'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Title'),
      '#default_value' => $this->getDefaultProperty('title'),
    ];

    // Start Date.
    $form['appointment_settings']['start_date'] = [
      '#type' => 'date',
      '#title' => $this->t('Start Date'),
      '#default_value' => $this->getDefaultProperty('start_date'),
    ];

    // End Date.
    $form['appointment_settings']['end_date'] = [
      '#type' => 'date',
      '#title' => $this->t('End Date'),
      '#default_value' => $this->getDefaultProperty('end_date'),
    ];

    // Exclusion Dates.
    $form['appointment_settings']['exclusion_dates'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Exclusion Dates'),
      '#description' => $this->t('Enter dates in format YYYY-MM-DD or intervals in format YYYY-MM-DD|YYYY-MM-DD, one per line. Ex. 2023-12-25 or 2023-12-25|2024-01-01'),
      '#default_value' => $this->getDefaultProperty('exclusion_dates'),
      '#rows' => 3,
    ];

    // Excluded Weekdays.
    $form['appointment_settings']['excluded_weekdays'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Excluded Weekdays'),
      '#options' => [
        'Mon' => $this->t('Mon'),
        'Tue' => $this->t('Tue'),
        'Wed' => $this->t('Wed'),
        'Thu' => $this->t('Thu'),
        'Fri' => $this->t('Fri'),
        'Sat' => $this->t('Sat'),
        'Sun' => $this->t('Sun'),
      ],
      '#default_value' => $this->getDefaultProperty('excluded_weekdays'),
    ];

    // Time intervals.
    $form['appointment_settings']['time_interval'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Time Intervals'),
      '#description' => $this->t('Enter time intervals in 24h format. One interval per line. Ex. 08:00|12:00. You can also add exceptions for specific days. Ex: 10:00|12:00(Friday)'),
      '#default_value' => $this->getDefaultProperty('time_interval'),
      '#required' => TRUE,
    ];

    // Slot Duration.
    $form['appointment_settings']['slot_duration'] = [
      '#type' => 'number',
      '#title' => $this->t('Slot Duration'),
      '#description' => $this->t('In Minutes'),
      '#default_value' => $this->getDefaultProperty('slot_duration'),
      '#min' => 1,
      '#required' => TRUE,
    ];

    // Seats per slot.
    $form['appointment_settings']['seats_slot'] = [
      '#type' => 'number',
      '#title' => $this->t('Seats Per Slot'),
      '#description' => $this->t('Number of bookings for the same slot'),
      '#default_value' => $this->getDefaultProperty('seats_slot'),
      '#min' => 1,
      '#required' => TRUE,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function prepare(array &$element, WebformSubmissionInterface $webform_submission = NULL) {
    // Ensure the library is attached only once.
    $element['#attached']['library'][] = 'webform_booking/webform_booking';

    $elementId = $element['#webform_key'];

    if (!isset($element['#attached']['drupalSettings']['webform_booking']['elements'])) {
      $element['#attached']['drupalSettings']['webform_booking']['elements'] = [];
    }

    // Use element ID as a key to ensure unique settings for each element.
    $element['#attached']['drupalSettings']['webform_booking']['elements'][$elementId] = [
      'formId' => $element['#webform'],
      'elementId' => $elementId,
      'startDate' => $element['#start_date'],
      'endDate' => $element['#end_date'],
    ];

    // Set up the HTML structure for the calendar and slots.
    $element['#description'] = '<div id="appointment-wrapper-' . $elementId . '"><div id="calendar-container-' . $elementId . '"></div><div id="slots-container-' . $elementId . '"></div></div>';

    $element['#type'] = 'textfield';
    $element['#attributes'] = ['id' => 'selected-slot-' . $elementId];
    $user = \Drupal::currentUser();
    if (!$user->hasPermission('edit any webform submission')) {
      $element['#attributes']['class'][] = 'hide';
    }

    return $element;
  }

}
